/***************************************************************************
                          ftable.cpp  -  description
                             -------------------
 Copyright (c) 2002 Tero Favorin (info@favorin.com)
 This program is distributed under the terms of the BSD License.
 See file COPYING for license details.
 ***************************************************************************/

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>

#include "ftable.h"

typedef KGlobal FGlobal;
typedef KIcon FIcon;

FTable::FTable(QWidget *parent, const char *name ) : QTable(parent,name) {
	init();
}
FTable::FTable(int numRows, int numCols, QWidget *parent, const char *name) : QTable(numRows,numCols,parent,name) {
	init();
}
FTable::~FTable(){
	delete padchar;
}
void FTable::init() {
	popup=NULL;
	autosort=false;
	sortedBy=0;
	sortAsc=true;
  colwidthsset=false;
  hidefocus=false;
	horizontalHeader()->installEventFilter( this );
  connect (horizontalHeader(),SIGNAL(sizeChange(int,int,int)),this,SLOT(slotSizeChange(int,int,int)) );
	padchar=new QChar('0');
}
void FTable::sortBy(int col, SortDirection sortDir) {
	if (autosort) {
		if (col>=0) {
			if (col==sortedBy) {
        if (sortDir==Swap) {
          if (sortAsc)
  					sortAsc=false;
  				else
  					sortAsc=true;
        }
			} else {
        horizontalHeader()->setLabel(sortedBy,QIconSet(),horizontalHeader()->label(sortedBy) );        
        switch (sortDir) {
          case Swap:
          case Ascending: sortAsc=true; break;
          case Descending: sortAsc=false; break;
        }
        sortedBy=col;
      }
		}
    QString icon="f_sortup";
    if (sortAsc)
      icon="f_sortdown";
		sortColumn(getDispColumn(sortedBy),sortAsc,true);
    horizontalHeader()->setLabel(sortedBy,FGlobal::iconLoader()->loadIcon(icon,FIcon::Small),
      horizontalHeader()->label(sortedBy) );
	}
}
void FTable::columnClicked(int col) {
	sortBy(col);
}
bool FTable::eventFilter( QObject *obj, QEvent *ev ) {
   if ( obj == horizontalHeader() && ev->type() == QEvent::MouseButtonRelease ) {
		 QMouseEvent *me=(QMouseEvent*)ev;
		 if (me->button()==RightButton && coltitles.count()>0) {
       if (popup->count()==0) {
          QValueList<int> olist;
          for (unsigned int i=0;i<coltitles.count();i++) {
            olist.append(getDispColumn(i));
          }
          qHeapSort(olist);
          QValueList<int>::iterator it;
          for ( it = olist.begin(); it != olist.end(); ++it ) {
            int i=getOrigColumn(*it);
           	int id=popup->insertItem(colnames[i]);
       		  popup->setItemChecked(id,colactives[i]);
      	  }
       }
			 popup->popup(mapToGlobal(me->pos()));
		   return true;	
		 }
   }
   return QTable::eventFilter( obj, ev );
}
void FTable::setColumns(QStringList t, QValueList<bool> a, QStringList n, QValueList<int> w) {
	Q_ASSERT(t.count()==a.count());
	coltitles=t;
	colactives=a;
	if (n.count()>0) {
		Q_ASSERT(n.count()==t.count());
		colnames=n;	
	} else
		colnames=t;
  if (!popup) {
		 popup = new FPopupMenu(this);			
		 connect (popup,SIGNAL(activated(int)),this,SLOT(columnDisplayChange(int)) );
	} else
		popup->clear();
  QStringList::Iterator it;
	int i=0;
  for ( it = coltitles.begin(); it != coltitles.end(); ++it ) {
		horizontalHeader()->setLabel(i,*it);
		i++;
	}
	setActives();
  if (w.count()>0) {
		Q_ASSERT(w.count()==t.count());
    QValueList<int>::ConstIterator it;
    colwidths.clear();
    int i=0;
    for ( it = w.begin(); it != w.end(); ++it ) {
      setColumnWidth(i++,*it);
      colwidths.append(*it);
    }
    colwidthsset=true;
  } else
    adjustAll();
  
}
void FTable::setActives() {
	for (unsigned int i=0;i<colactives.count();i++) {
		if (colactives[i])
			showColumn(getDispColumn(i));
		else
			hideColumn(getDispColumn(i));
	}
}
QValueList<int> FTable::colActivesInt() const {
  QValueList<int> temp;
  QValueList<bool>::ConstIterator it;
  for ( it = colactives.begin(); it != colactives.end(); ++it )
    temp.append(*it);
  return temp;
}
QValueList<int> FTable::colWidths() const {
  QValueList<int> temp;
  if (colwidthsset) {
    for (int i=0;i<numCols();i++) {
      int w=columnWidth(i);
      if (!w)
        if (colwidths.count()>(unsigned int)i)
          w=colwidths[i];  
      temp.append(w);    
    }
  }
  return temp;
}
void FTable::adjustAll() {
	for (unsigned int i=0;i<colactives.count();i++) {
		if (colactives[i])
			adjustColumn(getDispColumn(i));
	}
}
// Orig col
void FTable::columnDisplayChange(int id) {
	int col=popup->indexOf(id);
	Q_ASSERT((unsigned int)col<colactives.count());
	if (colactives[col])
		hideColumn(getDispColumn(col));
	else
		showColumn(getDispColumn(col));
}
// Disp col
void FTable::hideColumn(int col) {
	if (colactives.count()>0) {
		int ocol=getOrigColumn(col);
		colactives[ocol]=false;
 		popup->setItemChecked(popup->idAt(ocol),false);
	}
  while (colwidths.count()<(unsigned int)(col+1))
    colwidths.append(0);  
  colwidths[col]=columnWidth(col);
	QTable::hideColumn(col);
}
// Disp col
void FTable::showColumn(int col) {
	if (colactives.count()>0) {
		int ocol=getOrigColumn(col);
		colactives[ocol]=true;
 		popup->setItemChecked(popup->idAt(ocol),true);
	}
  QTable::showColumn(col);
  if (!columnWidth(col)) {
    if (colwidths[col])
      setColumnWidth(col,colwidths[col]);
    else
      setColumnWidth(col,100);    
  }
}
void FTable::swapColumns(int col1, int col2, bool swapHeader) {
  if (!columnMovingEnabled())
    return;

	QTable::swapColumns(col1,col2,swapHeader);
  int ocol1=getOrigColumn(col1);
	int ocol2=getOrigColumn(col2);
  if (!swapHeader) {
    colmap.insert(ocol1,col2);
  	colmap2.insert(col2,ocol1);
  	colmap.insert(ocol2,col1);
  	colmap2.insert(col1,ocol2);
  }
  if (popup!=NULL)
 		popup->clear();
}
int FTable::getOrigColumn(int dcol) {
	ColMap::ConstIterator it=colmap2.find(dcol);
	if (it!=colmap2.end())
		dcol=*it;
	return dcol;
}
int FTable::getDispColumn(int ocol) {
	ColMap::ConstIterator it=colmap.find(ocol);
	if (it!=colmap.end())
		ocol=*it;	
	return ocol;
}
void FTable::readConfig(FConfig *config, const QString &name, const QStringList &titles, bool a[], const QStringList &names) {
  QValueList<int> atemp, widths;
  QValueList<bool> actives;
  int len=titles.count();

  config->setGroup("FTable_"+name);
  atemp=config->readIntListEntry("ActiveColumns");
  QValueList<int>::ConstIterator it;
  for ( it = atemp.begin(); it != atemp.end(); ++it )
    actives.append(*it);
  widths=config->readIntListEntry("Widths");

  if (!actives.count()) {
    for (int i=0;i<len;i++)
    	actives.append(a[i]);
  }
  setColumns(titles,actives,names,widths);

  sortedBy=config->readNumEntry("SortColumn",0);
  sortAsc=config->readBoolEntry("SortOrder",true);
      
  QValueList<int> orig=config->readIntListEntry("ColumnOrder1");
  QValueList<int> disp=config->readIntListEntry("ColumnOrder2");
  if (orig.count()>0 && disp.count()>0 && orig.count()==disp.count()) {
    QValueList<int>::iterator it;
    QValueList<int>::iterator it2;
    it2=disp.begin();
    for ( it = orig.begin(); it != orig.end(); ++it ) {
      swapColumns(getDispColumn(*it),*it2,true);      
      ++it2;
    }
  }
    
}
void FTable::writeConfig(FConfig *config, const QString &name) const {
  config->setGroup("FTable_"+name);
  config->writeEntry("ActiveColumns",colActivesInt());
  config->writeEntry("Widths",colWidths());
  QValueList<int> orig,disp;
	ColMap::ConstIterator it;
  for ( it = colmap.begin(); it != colmap.end(); ++it ) {
    orig.append(it.key());
    disp.append(it.data()); 
  }
  config->writeEntry("ColumnOrder1",orig);	
  config->writeEntry("ColumnOrder2",disp);
  config->writeEntry("SortColumn",sortedBy);
  config->writeEntry("SortOrder",sortAsc);  
}
void FTable::slotSizeChange(int,int,int) {
  colwidthsset=true;
}
void FTable::paintFocus(QPainter *p,const QRect &cr) {
	if (!hidefocus) {
   	QTable::paintFocus(p,cr);
	}
}
QString FTable::pad(QString str) const {
	if (str.length()<2)
		return str.insert(0,padchar,2-str.length());
	return str;			
}

// FDateTableItem
FDateTimeTableItem::FDateTimeTableItem(FTable *table, EditType et, const QDateTime & d) :
	QTableItem (table, et, QString()) {
	setDateTime(d);
}
FDateTimeTableItem::~FDateTimeTableItem() {
}
QString FDateTimeTableItem::key() const {
	FTable *t=(FTable*)table();
	return QString::number(date.date().year())+t->pad(QString::number(date.date().month()))+t->pad(QString::number(date.date().day()))+
		t->pad(QString::number(date.time().hour()))+t->pad(QString::number(date.time().minute()))+t->pad(QString::number(date.time().second()));
}
void FDateTimeTableItem::setDateTime(const QDateTime &d) {
	date=d;
	setText(FGlobal::locale()->formatDateTime(d,true));	
}
QDateTime FDateTimeTableItem::dateTime() const {
 	return date;
}

// FDateItem
FDateTableItem::FDateTableItem(FTable *table, EditType et, const QDate & d) :
	QTableItem (table, et, QString()) {
	setDate(d);
}
FDateTableItem::~FDateTableItem() {
}
QString FDateTableItem::key() const {
	FTable *t=(FTable*)table();
	return QString::number(ddate.year())+t->pad(QString::number(ddate.month()))+t->pad(QString::number(ddate.day()));
}
void FDateTableItem::setDate(const QDate &d) {
	ddate=d;
	setText(FGlobal::locale()->formatDate(d,true));	
}
QDate FDateTableItem::date() const {
 	return ddate;
}

// FKeyTableItem
FKeyTableItem::FKeyTableItem(FTable *table, EditType et, const QString & str, void * i) :
	QTableItem (table, et, str) {
	_item=i;
}
FKeyTableItem::~FKeyTableItem() {
}
