/***************************************************************************
                          ftable.h  -  description
                             -------------------
 Copyright (c) 2002 Tero Favorin (info@favorin.com)
 This program is distributed under the terms of the BSD License.
 See file COPYING for license details.
 ***************************************************************************/



#ifndef FTABLE_H
#define FTABLE_H

#include <qwidget.h>
#include <qtable.h>
#include <qdatetime.h>
#include <qmap.h>

#include <kconfig.h>
#include <kpopupmenu.h>

/**
  *@author Tero Favorin
  */
typedef QValueList<bool> ActiveColumns;
typedef QValueList<int> ColWidths;

typedef KConfig FConfig;
typedef KPopupMenu FPopupMenu;

typedef QMap<int,int> ColMap;

class FTable : public QTable {
   Q_OBJECT
public:
  enum SortDirection{Swap,Ascending,Descending};

	FTable(QWidget *parent=0, const char *name=0);
	FTable(int numRows, int numCols, QWidget *parent=0, const char *name=0);
	~FTable();
	bool autoSort() const {return autosort;}
	void setAutoSort(bool a) {autosort=a;}
	void setColumns(QStringList titles, QValueList<bool> actives, QStringList names=QStringList(), QValueList<int> widths=QValueList<int>());
  void readConfig(FConfig *config, const QString &name, const QStringList &titles, bool actives[], const QStringList &names);
  void writeConfig(FConfig *config, const QString &name) const;
  QStringList colTitles() const {return coltitles;}
	QValueList<bool> colActives() const {return colactives;}
	QValueList<int> colActivesInt() const;
  QValueList<int> colWidths() const;
  bool colWidthsSet() const {return colwidthsset;}
  void setActives();
	void adjustAll();
	virtual void paintFocus(QPainter *,const QRect &);
	void setHideFocus(bool f) {hidefocus=f;}
	bool hideFocus() const {return hidefocus;}
	QString pad(QString) const;
	int getOrigColumn(int dcol);
	int getDispColumn(int ocol);
public slots:
	void sortBy(int col=-1, SortDirection sortDir=Swap);
	virtual void swapColumns(int col1, int col2, bool swapHeader=TRUE);
	virtual void hideColumn(int col);
	virtual void showColumn(int col);  
protected:
	bool eventFilter( QObject *obj, QEvent *ev );
protected slots:
	virtual void columnClicked(int col);
  void slotSizeChange(int,int,int);
private:
	void init();
	
	QChar *padchar;
	QStringList coltitles;
	QStringList colnames;
	ActiveColumns colactives;
	FPopupMenu *popup;

	ColMap colmap; // Original column to display column	
	ColMap colmap2; // Display column to original column
  ColWidths colwidths; // Column widths for showColumn (work around for qt bug, that sets column width to 0 when hiding.)
    
	bool autosort;
	int sortedBy;
	bool sortAsc;
	bool hidefocus;
  bool colwidthsset;
private slots:
	void columnDisplayChange(int col);
};

class FDateTimeTableItem : public QTableItem {

public:
	FDateTimeTableItem(FTable *table, EditType et, const QDateTime & date);
	~FDateTimeTableItem();
	void setDateTime(const QDateTime &date);
	QDateTime dateTime() const;
	virtual QString key() const;
private:
	QDateTime date;
};

class FDateTableItem : public QTableItem {

public:
	FDateTableItem(FTable *table, EditType et, const QDate & date);
	~FDateTableItem();
	void setDate(const QDate &d);
	QDate date() const;
	virtual QString key() const;
private:
	QDate ddate;
};

class FKeyTableItem : public QTableItem {

public:
  FKeyTableItem(FTable *table, EditType et, const QString & str, void * item);
	~FKeyTableItem();

  void setItem(void *i) {_item=i;}
  void* item() const {return _item;}
private:
  void * _item;
};

#endif
