/***************************************************************************
                          kexconvert.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Tero Favorin
    email                : tero@favorin.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qvbox.h>
#include <qvgroupbox.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qwhatsthis.h>
#include <qlabel.h>

#include <klineedit.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kcombobox.h>
#include <kdialog.h>
#include <knumvalidator.h>

#include <fpixmapwidget.h>

#include "kexconvert.h"
#include "kexdataengine.h"
#include "kexsource.h"

KExConvert::KExConvert(QWidget * parent, const char * name, WFlags f):QWidget(parent,name,f) {
QGridLayout *layout =new QGridLayout(this,5,3,KDialog::marginHint(),KDialog::spacingHint());
layout->setRowStretch(4, 5);
layout->setColStretch(1, 1);

from=new QLabel(i18n("From"),this);
layout->addWidget(from,0,0);
fromCombo=new KComboBox(this);
QWhatsThis::add( fromCombo, i18n("This is the currency which you want to convert to another currency.") );
layout->addWidget(fromCombo,0,1);
connect (fromCombo,SIGNAL(activated(int)),this,SLOT(updateRate(int)));

QHBox *fromBox=new QHBox(this);
fromBox->setSpacing(KDialog::spacingHint());
layout->addWidget(fromBox,1,1);
input=new KLineEdit(fromBox);
input->setAlignment(AlignRight);
KFloatValidator *fval=new KFloatValidator(this);
fval->setAcceptLocalizedNumbers (true);
input->setValidator(fval);
input->setMaxLength(16);
QWhatsThis::add( input, i18n("This is the amount of currency units you want to convert to another currency.") );
connect (input,SIGNAL(textChanged(const QString&)),this,SLOT(update(const QString&)));
fromCur=new QLabel(fromBox);
QWhatsThis::add( fromCur, i18n("Currency code of the currency being converted.") );

to=new QLabel(i18n("To"),this);
layout->addWidget(to,2,0);
toCombo=new KComboBox(this);
QWhatsThis::add( toCombo, i18n("This is the currency which you are converting to.") );

layout->addWidget(toCombo,2,1);
connect (toCombo,SIGNAL(activated(int)),this,SLOT(updateRate(int)));

QHBox *toBox=new QHBox(this);
toBox->setSpacing(KDialog::spacingHint());
layout->addWidget(toBox,3,1);
output=new QLabel("0.00",toBox);
output->setAlignment(AlignRight);
QWhatsThis::add( output, i18n("This is the amount that has been converted from another currency.") );
toBox->setStretchFactor(output,5);
toCur=new QLabel(toBox);
QWhatsThis::add( toCur, i18n("Currency code of converted currency.") );

QVGroupBox *rateBox=new QVGroupBox(i18n("Exchange rate"),this);
rateBox->setMargin(KDialog::marginHint());

QHBox *rateBox2=new QHBox(rateBox);
rateBox2->setSpacing(KDialog::spacingHint());
rate=new QLabel(rateBox2);
QWhatsThis::add( rateBox2, "<p>"+i18n("This is the exchange rate used in currency conversion.")+"</p>"+
  i18n("Rate codes:")+
  "<table><tr><td><img src=\"reliable.png\"></td><td>"+i18n("The displayed rate is reliable")+
  "</td></tr><tr><td><img src=\"conversion.png\"></td><td>"+i18n("The displayed rate is a fixed conversion rate.")+
  "</td></tr><tr><td><img src=\"calculated.png\"></td><td>"+i18n("The displayed rate is calculated via another currency (USD or EUR).")+
  "</td></tr><tr><td><img src=\"crossconversion.png\"></td><td>"+i18n("The displayed rate is cross conversion rate between national units of euro. The rate is only indicative and is not used in actual calculation.")+
  "</td></tr><td><tr><td><img src=\"unreliable.png\"></td><td>"+i18n("The displayed rate may be unreliable.")+"</td></tr></table>");
rateIcon=new FPixmapWidget(QPixmap(),rateBox2);
rateIcon->setFrameStyle(QFrame::Box | QFrame::Sunken);
rateIcon->setMargin(3);
QLabel *pad=new QLabel(rateBox2);
rateBox2->setStretchFactor(pad,10);

rateText2=new QLabel(i18n("Rate as of"),rateBox);
rateDate=new QLabel(rateBox);
QWhatsThis::add( rateDate, i18n("This is the date of exchange rate.")+"\n\n"+i18n("For fixed conversion rates the date when the rate was adopted is shown.") );
layout->addMultiCellWidget(rateBox,0,3,2,2);

source=new KExSource(this);
layout->addMultiCellWidget(source,4,4,0,2,AlignLeft | AlignBottom);

input->setFocus();
}
KExConvert::~KExConvert(){
}
void KExConvert::setOptions(KExDataEngine *eng, bool e) {
	QPixmap flag;	

	engine=eng;
	oldeuro=e;

	source->setSource(engine);
	fromCombo->clear();
	toCombo->clear();
	list=engine->getCurrencies(oldeuro);
  for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
		flag=engine->getFlag(*it);
	  fromCombo->insertItem(flag, engine->getCurrencyName(*it));
		toCombo->insertItem(flag, engine->getCurrencyName(*it));
  }
}
void KExConvert::setExchange(const QString &to, const QString &from) {
	toCombo->setCurrentItem(list.findIndex(to));
	fromCombo->setCurrentItem(list.findIndex(from));	
	updateRate(-1);
}
QString KExConvert::getFrom() const {
	return engine->getCode(list,fromCombo->currentItem());
}
QString KExConvert::getTo() const {
	return engine->getCode(list,toCombo->currentItem());
}
void KExConvert::setInput(const QString &s){
	bool ok;
	KGlobal::locale()->readNumber(s,&ok);
	if (ok) {
		input->setText(s);
		update(s);
	}
}
void KExConvert::updateRate(int) {
	fromCur->setText(getFrom());
	toCur->setText(getTo());
	KExData exrate=engine->getRate(getTo(),getFrom());
	rate->setText(engine->round(exrate.rate));
	rateDate->setText(KGlobal::locale()->formatDate(exrate.date,true));
  rateIcon->setPixmap(engine->getConfidenceIcon(exrate.confidence));
	update(QString::null);
}
void KExConvert::update(const QString& in) {
	bool ok;
	QString str=in;
	if (str.isNull())
		str=input->text();
	double amount=KGlobal::locale()->readNumber(str,&ok);
	if (!ok)
		amount=0;
	output->setText(KGlobal::locale()->formatNumber(engine->convert(getTo(),getFrom(),amount),2));			
}
QString KExConvert::getOutput() const {
  return output->text();
}
