/***************************************************************************
                          kexdatasource.h  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Tero Favorin
    email                : tero@favorin.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KEXDATAENGINE_H
#define KEXDATAENGINE_H

#include <qobject.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qptrlist.h>
#include <qpixmap.h>
#include <qdatetime.h>
#include <dcopobject.h>

#include "kexdatasource.h"

/**
  *@author Tero Favorin
  */

typedef double Rate;
enum Confidence{None,Conversion,Reliable,Calculated,CrossConversion,Unreliable};
enum Status{Ready,Transfering,Error};

class KExData {
public:
	KExData() {rate=0;date=QDate();confidence=None;}
	KExData(const Rate &r, const QDate &d, Confidence c) {rate=r;date=d;confidence=c;}
Rate rate;
QDate date;
Confidence confidence;
};

typedef QMap<QString,QString> CurrencyNames;
typedef QMap<QString,KExData> ExData;
typedef QPtrList<KExDataSource> DataSources;

class KExDataEngine : public QObject, public DCOPObject {
Q_OBJECT
K_DCOP

friend class KExDataRetriever;
friend class KExDataSource;
friend class KExPacificSupplementary;

public: 
	KExDataEngine(QWidget *w);
	virtual ~KExDataEngine();
	/** Get rate between base and target currencies */
	const KExData getRate(const QString &target, const QString& base) const;
	/** Set calculation precision */
	void setPrecision(int p) {precision=p;}
	/** Get currency code from the list */
	static QString getCode(const QStringList &list, int pos);
	/** Get rounded rate */
	QString round(Rate rate) const;
  /** Add data source */
  void addSource(KExDataSource *source);
  /** Get default data source */
  KExDataSource* getSource();
  /** Refresh all data sources */
  void refresh();
  /** Get parent widget */
  QWidget* getParent() const {return parent;}
	/** Parse QDate from YYYY-MM-DD */
	static const QDate parseDate(const QString &str, const QChar &sep);  
k_dcop:
	/** Get rate between base and target currencies */
	const double getExchangeRate(const QString &target, const QString &base) const {return getRate(target,base).rate;}
	/** Get date of rate between base and target currencies */
	const QDate getExchangeRateDate(const QString &target,const QString &base) const {return getRate(target,base).date;}
	/** Get rate confidence between base and target currencies */
	const int getExchangeRateConfidence(const QString &target, const QString &base) const {return getRate(target,base).confidence;}
	/** Get confidence pixmap. */
	const QPixmap getConfidenceIcon(int c) const;
  /** Get rounded rate */
	QString roundRate(double rate) const {return round(rate);}
	/** Get currency name from the code. */
	const QString getCurrencyName(const QString &code) const;
	/** Convert from target currency to base currency */
	double convert(const QString &target, const QString &base, double amount) const;
	/** Return list of available currency codes. */
	const QStringList getCurrencies(bool showOldEuroCurrencies=true) const;
	/** Get flag ot the country for the currency. */
	const QPixmap getFlag(const QString &code);
  /** Get all rates as text */  
  QString getAllRates(const QString &base, bool unitToBase, bool showEuro) const;
signals:
	/** Status message about the data transfer */
	void status(int type, const QString &);
	/* Emitted when data transfer has completed */
	void dataReady(bool ok);

protected:
	/** Add currency to available currencies list. */
	bool addCurrency(const QString &code);
	/** Data currency exchange rate */
	void addRate(const QString &target, const QString &base, Rate rate, const QDate &date, Confidence confidence);
  /** Emit transfer status */
  void emitStatus(int type, const QString &);
	/** Remove currency rate. */
	void removeRate(const QString &key);
  /** Clear old exchange rates */
  void clearOldRates();
private slots:
  void sourceReady();
private:
	/** Get rate to USD. */
	KExData toUSD(QString code) const;
	/** Get Exchange rate key for ExData. */
	const QString getKey(const QString &target, const QString &base) const;
  /** Get target rate from the key.
    If the last rate for the target currency is removed, the currency is removed from the list.
   */
  const QString getTarget(const QString &key) const;
	/** Round doubles. */
	Rate round(Rate doValue, int nPrecision, bool upgrade=true) const;
	/** Read data from XML file. */
	void readData();
	/** Store data to XML file.	*/
	void storeData() const;  
	/** Display widget */
	QWidget *parent;
	/** Default precision */
	int precision;
	/** High precision */
	int highPrecision;
	/** Currency code & name pairs. */
	CurrencyNames names;
  /** Data sources */
  DataSources sources;	
	/** List of available currencies. */
	QStringList list;
	/** List of old native currencies that have switched to Euro. */
	QStringList eurolist;
	/** Exchange rates data. */
	ExData data;
	/** Default (empty) pixmap for flags */
	QPixmap defaultFlag;

  int runningSource;
};

#endif
