/***************************************************************************
                          kexdataretriever.cpp  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by Tero Favorin
    email                : tero@favorin.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qtextstream.h>
#include <qregexp.h>

#include <klocale.h>
#include <kmessagebox.h>

#include "kexdatasource.h"
#include "kexdataretriever.h"

KExDataRetriever::KExDataRetriever(const QString& name,KExDataSource *src) : QObject(0){
  _name=name;
  engine=src->engine;
  connect(this,SIGNAL(retrieverReady(bool)),src,SLOT(retrieverReady(bool)) );
 	datain=NULL;
  streamin=NULL;
}
KExDataRetriever::~KExDataRetriever(){  
  clear();
}
void KExDataRetriever::clear() {
	delete streamin;
  streamin=0;
	delete datain;
  datain=0;
}  
bool KExDataRetriever::addCurrency(const QString &code) {
  return engine->addCurrency(code);
}  
void KExDataRetriever::addRate(const QString &target, const QString &base, Rate rate, const QDate &date, Confidence confidence) {
  engine->addRate(target,base,rate,date,confidence);
}  

void KExDataRetriever::transferData(KIO::TransferJob *job) {
 clear();
 datain=new QByteArray();
 streamin=new QDataStream(*datain,IO_WriteOnly);
 connect( job, SIGNAL( result( KIO::Job * ) ), this, SLOT( slotResult( KIO::Job * ) ) );
 connect( job, SIGNAL( data( KIO::Job *, const QByteArray &  ) ), this, SLOT( slotData( KIO::Job *, const QByteArray &  ) ) );
 connect (job, SIGNAL(infoMessage ( KIO::Job *, const QString & ) ), this, SLOT (slotMessage ( KIO::Job *, const QString & ) ) );
 engine->emitStatus(Transfering,i18n("Connecting to server"));
}
void KExDataRetriever::slotResult( KIO::Job *job) {
	bool ok=false;
	if (job->error()) {
		job->showErrorDialog(engine->getParent());
		engine->emitStatus(Error,i18n("Error retrieving currency data"));
	}
	else {
		KIO::TransferJob *tjob=(KIO::TransferJob*)job;
	  if (tjob->isErrorPage ()) {
			KMessageBox::error (engine->getParent(), i18n("Server returned an error"));
			engine->emitStatus(Error,i18n("Error retrieving currency data"));
		}	else {
      QTextStream ts( *datain, IO_ReadOnly );
			if (readData(ts)) {
	  		engine->emitStatus(Ready,i18n("Currency data retrieved successfully"));
        ok=true;
			} else
	  		engine->emitStatus(Error,i18n("Could not read currency data from")+" "+name());
		}
	}
  clear();
	emit retrieverReady(ok);
}
void KExDataRetriever::slotMessage ( KIO::Job *, const QString & m) {
	QString msg=m;
	msg=msg.replace(QRegExp("<b>"),"");
	msg=msg.replace(QRegExp("</b>"),"");
	engine->emitStatus(Transfering,msg);
}
void KExDataRetriever::slotData ( KIO::Job *, const QByteArray &d) {
	if (d.size()>0)
		*streamin << d;
}
QString KExDataRetriever::getText(QString txt) {
	int x,y;
  x=txt.find('<');
  while (x>=0) {
    y=txt.find('>');
    txt.remove(x,y-x+1);
    x=txt.find('<');
  }
	txt.replace("&nbsp;"," ");
	return txt;
}  
