/***************************************************************************
                          kextable.cpp  -  description
                             -------------------
    begin                : Tue Mar 5 2002
    copyright            : (C) 2002 by Tero Favorin
    email                : tero@favorin.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qpixmap.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qvbuttongroup.h>
#include <qradiobutton.h>
#include <qwhatsthis.h>

#include <kstandarddirs.h>
#include <klocale.h>
#include <kdialog.h>
#include <kcombobox.h>
#include <kglobal.h>

#include <ftable.h>

#include "kexdataengine.h"
#include "kexsource.h"
#include "kexrateitem.h"
#include "kextable.h"

#define CURRENCYCOL 0
#define CODECOL 1
#define RATECOL 2
#define TYPECOL 3
#define DATECOL 4

#define CONFIGNAME "Rate table"

KExTable::KExTable(QWidget * parent, KConfig *config, const char * name, WFlags f):QWidget(parent,name,f) {
	QVBoxLayout *layout =new QVBoxLayout(this,KDialog::marginHint(),KDialog::spacingHint());

	QHBox *baseBox=new QHBox(this);
	baseBox->setSpacing(KDialog::spacingHint());
	layout->addWidget(baseBox);
	base = new QLabel(i18n("Base currency"),baseBox);
	baseCombo=new KComboBox(baseBox);
	QWhatsThis::add( baseCombo, i18n("This is the base currency to which all currencies in the table are compared to.") );
  QWhatsThis::add(base,QWhatsThis::textFor(baseCombo));
  connect (baseCombo,SIGNAL(activated(int)),this,SLOT(updateBase(int)));

	QVButtonGroup *buttonGroup=new QVButtonGroup(baseBox);
	buttonGroup->setFrameStyle(QFrame::NoFrame);
  buttonGroup->setInsideMargin(0);
  buttonGroup->setInsideSpacing(0); 
	unitBase=new QRadioButton(i18n("Unit/Base"),buttonGroup);	
	QWhatsThis::add( unitBase, i18n("Display rates as currency unit / base currency. This is called the volume notation.") );
	connect ( unitBase,SIGNAL(toggled(bool)),this,SLOT(unitToggle(bool)) );

	baseUnit=new QRadioButton(i18n("Base/Unit"),buttonGroup);	
	QWhatsThis::add( baseUnit, i18n("Display rates as base currency / currency unit. This is called the price notation.") );
	connect ( baseUnit,SIGNAL(toggled(bool)),this,SLOT(unitToggle(bool)) );

	table=new FTable(1,5,this);
	layout->addWidget(table);
	table->setSelectionMode(QTable::NoSelection);
	table->setLeftMargin( 0 );
  table->verticalHeader()->hide();
  table->horizontalHeader()->setMovingEnabled(false);
  QStringList titles;
  titles.append(i18n("Currency"));
  titles.append(i18n("Code"));
  titles.append(i18n("Exchange rate"));  
  titles.append(i18n("Type"));
  titles.append(i18n("Date"));  
  /*QValueList<bool> actives;
  actives.append(true);
  actives.append(true);
  actives.append(true);    
  actives.append(false);
  actives.append(false);  
  table->setColumns(titles,actives);*/
  bool actives[]={true,true,true,false,false};
  table->readConfig(config,CONFIGNAME,titles,actives,QStringList());
  
	source=new KExSource(this);
	layout->addWidget(source);

}
KExTable::~KExTable(){
}
void KExTable::writeConfig(KConfig *config) {
  table->writeConfig(config,CONFIGNAME);
}  
void KExTable::setOptions(KExDataEngine *eng, bool e) {
	QPixmap flag;

	engine=eng;
	oldeuro=e;
	source->setSource(engine);
	list=engine->getCurrencies(oldeuro);
	table->setNumRows(list.count());
	baseCombo->clear();
	int row=0;
  for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
		flag=engine->getFlag(*it);
	  table->setItem(row,CURRENCYCOL,new QTableItem(table,QTableItem::Never,engine->getCurrencyName(*it),flag));
	  table->setItem(row++,CODECOL,new QTableItem(table,QTableItem::Never,*it));
	  baseCombo->insertItem(flag, engine->getCurrencyName(*it));
  }
	table->adjustColumn(CURRENCYCOL);
	table->adjustColumn(CODECOL);
}
QString KExTable::getBase() const {
	return engine->getCode(list,baseCombo->currentItem());
}
void KExTable::setBase(const QString &base) {
	baseCombo->setCurrentItem(list.findIndex(base));	
	updateBase(-1);
}
void KExTable::setUnitBase(bool ub) {
	if (ub)
		unitBase->setChecked(true);
	else
		baseUnit->setChecked(true);
}
bool KExTable::getUnitBase() const {
	return unitBase->isChecked();
}
void KExTable::unitToggle(bool) {
 	updateBase(0);
}
void KExTable::updateBase(int) {
	QString base=getBase();
	int row=0;
	KExData r;

  for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
		if (unitBase->isChecked())
			r=engine->getRate(*it,base);
		else
			r=engine->getRate(base,*it);
	  table->setItem(row,RATECOL,new KExRateItem(table,QTableItem::Never,engine->round(r.rate)));
    table->setItem(row,TYPECOL,new QTableItem(table,QTableItem::Never,QString(),engine->getConfidenceIcon(r.confidence)));
    table->setItem(row++,DATECOL,new QTableItem(table,QTableItem::Never,KGlobal::locale()->formatDate(r.date,true)));
  }	
	table->adjustColumn(RATECOL);
	table->adjustColumn(TYPECOL);
	table->adjustColumn(DATECOL);   
}
